using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;


namespace BallDroppingOnXBox
{
    class Circle2D
    {
        private Vector3 center;
		private float radius;
		private int kNumPts;
        private VertexPositionColor[] vertices;
        private VertexDeclaration vertexDecl;
		
		public Circle2D(GraphicsDevice d, Vector3 p, float r, int resolution)
		{
            vertexDecl = new VertexDeclaration(d, VertexPositionColor.VertexElements);
			center = p;
			radius = r;		
			kNumPts = resolution;
			vertices = new VertexPositionColor[kNumPts+2];
			recalculate();
		}

	
		
		public void Draw(GraphicsDevice d) 
		{
            d.VertexDeclaration = vertexDecl;

            // Triangle Fan is not working yet ... why?
            d.RenderState.FillMode = FillMode.Solid;
            d.DrawUserPrimitives(PrimitiveType.TriangleFan, vertices, 0, kNumPts);
            d.RenderState.PointSize = 5.0f;
            d.DrawUserPrimitives(PrimitiveType.PointList, vertices, 0, kNumPts);
		}

		public void setCenter(Vector3 p) 
		{
			center = p;
			recalculate();
		}

		public Vector3 getCenter() { return center; }

		public void setRadius(float r)
		{
			radius = r;
			recalculate();
		}
		public float getRadius() { return radius; }

		/// <summary>
		/// Recalculates all vertices
		/// </summary>
		private void recalculate() 
		{		
			float theta = (2.0f * (float)Math.PI) / kNumPts;
			vertices[0].Position = new Vector3(center.X, center.Y, 0.0f);
			vertices[0].Color = Color.Cyan;

			for (int i = 0; i < kNumPts; i++) 
			{
				vertices[i+1].Position = new Vector3(
					center.X + (radius * (float)Math.Sin(-i*theta)), 
					center.Y + (radius * (float)Math.Cos(-i*theta)), 
					0.0f); 
				vertices[i+1].Color = Color.Magenta;
			}
			vertices[kNumPts+1] = vertices[1];
			
		}
	}
}
